from flask import request

class MethodSwitch:
    def __init__(self):
        self.get = False
        self.post = False
        self.put = False
        self.delete = False
        self.options = False
        self.header = False

class Route:
    """
    Consolidate all route logic into one logic.

    The goal is to be able to define an object :Route: that will
    handle endpoints on multiple methods.
    """

    isblank = MethodSwitch()
    def __init__(self):
        pass

    def _setmethod(self, methods, method):
        if not getattr(self.isblank, method.lower()):
            methods += [method]

    def getmethods(self):
        methods = []
        for m in ['GET', 'POST', 'PUT', 'DELETE', 'OPTIONS', 'HEADER']:
            self._setmethod(methods, m)
        return methods

    def get(self):
        self.isblank.get = True
        pass

    def post(self):
        self.isblank.post = True
        pass

    def put(self):
        self.isblank.put = True
        pass

    def delete(self):
        self.isblank.delete = True
        pass

    def options(self):
        self.isblank.options = True
        pass

    def header(self):
        self.isblank.header = True
        pass

    def sethttp(self):
        method = request.method
        if method and not getattr(self.isblank, method):
            outbound_fxn = getattr(self, method)
            outbound_fxn()



class AppNotDefined(Exception):
    """All routes will fail if app is not defined"""
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class HttpMethodConflict(Exception):
    """Defined HTTP methods with defined function once"""
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message



import json
from flask import request
from .route import Route
from .exceptions import AppNotDefined, HttpMethodConflict

class Froi(Route):
    """
    Template for all routes to be created for APIs.

    A Flask wrapper that installs routes as defined from a class.
    It will accept :app: as paremeter that will be the server's
    context and will be used to set route.
    """

    def __init__(self, app, component_name, prefix=''):
        self.component_name = component_name
        self.prefix = prefix
        if app.add_url_rule is None:
            raise AppNotDefined('Sent `app` is not valid')

        self.app = app
        self.method = []

    def route(self, url='', **kwargs):
        """Wrap server method's route"""
        add = self.app.add_url_rule
        add('{}{}'.format(self.prefix, url),
            methods=self.getmethods(),
            **kwargs)

        # handle forward slash
        if self.prefix is not '' or url is not '':
            add('{}{}/'.format(self.prefix, url),
                methods=self.getmethods(),
                **kwargs)

        self.method = []

    def install(self):
        """Attach routes to defined app"""
        self.setall().route(view_func=self.sethttp)

    def _check_methods(self, method):
        if method in self.method:
            raise HttpMethodConflict('Multiple {} in methods'.format(method))

        if self.method is not None:
            self.method += [method]
            return

        self.method = [method]

    def setall(self):
        self.method = ['GET', 'POST', 'PUT',
                       'DELETE', 'OPTIONS', 'HEADER']
        return self

    def setget(self):
        self._check_methods('GET')
        return self

    def setpost(self):
        self._check_methods('POST')
        return self

    def setput(self):
        self._check_methods('PUT')
        return self

    def setdelete(self):
        self._check_methods('DELETE')
        return self

    def setoptions(self):
        self._check_methods('OPTIONS')
        return self

    def setheader(self):
        self._check_methods('HEADER')
        return self
